import lib.reg as reg
from lib.parsing.parser import Parser, ParsingError
import signal
from lib.checker import transform, dfa_transform, nfa_transform, check_task, check_alphabets, check_empty

class Language():
    def __init__(self, string: str, task: str):
        self.string = string
        self.task = task
        self.dfa = None
        self.minimal = None
        self.inf = "prázdný"
        self.example = ""
        self.gen_lang_html()

    def gen_lang_html(self):
        parser = Parser()
        dfa = dfa_transform(self.string, self.task)
        self.minimal = parser.dfa_to_str(dfa.minimize().canonize())

        empty = dfa.is_empty()
        if empty.inf is not None:
            self.inf = "nekonečný" if empty.inf else "konečný"
            self.example = empty.counterexample

        self.dfa = parser.dfa_to_str(dfa.canonize())

    def gen_is_task(self, task: str) -> str:
        teacher = "NFA" if self.task == "EFA" else self.task
        return f"{teacher}-{task}-N:{self.dfa}"


class WebChecker():
    def __init__(self, student_string: str, task: str):
        self.student_string = student_string
        self.task = task
        self.eq = None
        self.ok = False

    def dfa_task(self, teacher_type, teacher_string, task, student_string):
        try:
            student_solution = transform(student_string, task)
            student_dfa = dfa_transform(student_string, task)
            teacher_dfa = dfa_transform(teacher_string, teacher_type)

        except ParsingError as ex:
            return(ex.args)

        if teacher_dfa.is_empty() and student_dfa.is_empty():
            self.ok = True
            return ""

        alphabets = check_alphabets(student_alpha=student_dfa.characters,
                                    teacher_alpha=teacher_dfa.characters, task=task)
        if alphabets != "":
            return alphabets

        task_solved = ""
        if isinstance(student_solution, reg.DFA) or isinstance(student_solution, reg.NFA):
            task_solved = check_task(student_solution, task)

        self.eq = reg.DFA.is_equivalent(student_dfa, teacher_dfa)

        if task_solved == "" and self.eq:
            self.ok = True

        return task_solved

    def compare(self, teacher_string: str, teacher_type: str):
        #signal.alarm(50) #
        try:
            return self.dfa_task(teacher_type=teacher_type, teacher_string=teacher_string,
                                 task=self.task, student_string=self.student_string)

        except Exception as ex:
            print("Error inside of web checker:", ex.args)


    def convert(self, student_type):
        #signal.alarm(50)

        try:
            parser = Parser()

            if self.task in {"DFA", "TOT", "MIN"}:
                dfa = dfa_transform(self.student_string, student_type)

                if self.task == "DFA":
                    return parser.dfa_to_str(dfa)
                if self.task == "TOT":
                    return parser.dfa_to_str(dfa.total())
                if self.task == "MIN":
                    return parser.dfa_to_str(dfa.minimize().canonize())
                #if canonize: TODO checkbox
                #    return parser.dfa_to_str(automaton.canonize())

            nfa = nfa_transform(self.student_string, student_type)
            if self.task == "EFA":
                return parser.nfa_to_str(nfa)

            nfa = nfa.eliminate_epsilon()
            if self.task == "NFA":
                return parser.nfa_to_str(nfa)

            if self.task == "GRA":
                return parser.reggrammar_to_str(nfa.nfa_to_reggrammar().eliminate_useless())

            if self.task == "REG":
                return self.student_string if student_type == "REG" else \
                        "Tohle není pěkný typ převodu. Nechcete si místo regulárního výrazu vybrat něco jiného?"
                #return parser.regex_to_str(automaton.nfa_to_regex())

        except ParsingError as ex:
            return("Chyba při parsování.")

        except Exception as ex:
            print("Error inside of web checker:", ex.args)